
from scipy import linalg
import numpy as np
from scipy import signal
from numpy import pi

def Matrices(m,c,k):
    #Crea las matrices del tamaño deseado
    n = m.shape[0] #GDL
    M = np.zeros((n,n))
    C = np.zeros((n,n))
    K = np.zeros((n,n))
    #Rellena las matrices (las matrices C y K se rellenan con un bucle, menos el termio del final):
    M = np.diag(m)
    for i in range(0,n-1):
        j=i+1
        C[i][i]=c[i]+c[j]
        C[i][j]=-c[j] 
        C[j][i]=-c[j]
        K[i][i]=k[i]+k[j]
        K[i][j]=-k[j] 
        K[j][i]=-k[j]
    C[n-1][n-1]=c[n-1]
    K[n-1][n-1]=k[n-1]
    return M,C,K
    
def mat2ss(M,C,K,Posic,Veloc,Acel):
    #GDL y matriz modelo vacia:
    n = M.shape[0]
    Z = np.zeros((n,n))
    #Matriz identidad:
    Im = np.eye(n)
    #Matriz iversa de M:
    M_Inv = linalg.inv(M)
    #Axis= (1 ó 0) dependiendo de si quiero fila 1 ó columna 0
    As1 = np.concatenate((Z, Im), axis=1)
    As2 = np.concatenate(((-M_Inv@K), (-M_Inv@C)), axis=1)
    As = np.concatenate((As1, As2), axis=0)
    Bs =np.concatenate((Z,M_Inv), axis=0)
    #Determinar el tamaño de filas de Cs:
    Tam = 0
    for a in range(0,n):
       Sum = Posic[a]+Veloc[a]+Acel[a]
       Tam = Tam+Sum
    #Completa Cs con las diferentes salidas que se buscan
    #Empieza por crear las matrices finales vacias.
    Cs =  np.zeros((Tam,n*2))
    Ds = np.zeros((Tam,n)) #Ds tendrá las mismas filas que Cs y columnas que Bs
    Cont=0 #Contador para ir pasando de una fila a la siguiente
    for a in range(0,n): #Posiciones
        if Posic[a] == 1:
            Cs[Cont][a] = Posic[a]
            Cont=Cont+1
    
    for a in range(0,n): #Velocidades
        if Veloc[a] == 1:
            Cs[Cont][a+n] = Veloc[a]
            Cont=Cont+1

    for a in range(0,n): #Aceleraciones
        if Acel[a] == 1:
            Cs[Cont][:] = As2[a][:]
            Ds[Cont][:] = Bs[a+n][:]
            Cont=Cont+1
    sys = signal.StateSpace( As , Bs , Cs , Ds )
    return sys

def simT(sys,F,dT,T,ws,Desf,X0): 
    n = F.shape[0]   #numero de grados de libertad
    Ts = np.arange( 0 ,  T , dT)
    #Crea el vector de fuerzas que ira en el sistema:
    num_elem = Ts.shape[0]
    Fs = np.zeros((num_elem,n))
    for a in range(0,n):#Fuerzas
        Fs [:,a]= F[a]*np.sin(ws[a]*Ts+Desf[a])
    T_out, y, x= signal.lsim(sys, Fs, Ts, X0)
    return T_out,y


#Aqui empeza la parte de calculo modal-----------

def calc_modal(M,C,K):
    #GDL y matriz modelo vacia:
    n = M.shape[0]
    Z = np.zeros((n,n))
    #Valores Propios no amortiguados w y vr---
    w,vr = linalg.eig(K, M)
    #Autovalores---
    w = w.real # Elimina la parte imaginaria
    w = np.sqrt(w) # pasa w2 a w
    w = w/(2*pi) # transformarlo a frecuencias propias en Hz dividiendo por 2π
    #Autovectores---
      #Valores Propios amortiguados:
    A1 = np.concatenate((C,M), axis=1)#Lo primero será reducir el orden del sistema pues ahora: (sr2 M + sr C + K) vc = 0;
    A2 = np.concatenate((M,Z), axis=1)
    A = np.concatenate((A1,A2), axis=0)
    B1 = np.concatenate((K,Z), axis=1)
    B2 = np.concatenate((Z,-M), axis=1)
    B = np.concatenate((B1,B2), axis=0)
    sr,q = linalg.eig(-B, A)
    #Autovalores---
    wr = np.abs(sr) # saca el modulo del numero complejo
    wr = wr/(2*pi) # transformarlo a frecuencias propias en Hz dividiendo por 2π
    sr = sr/(2*pi)
    #factores de amortiguamiento crítico---
    dr = (-(np.real(sr)))/(np.abs(sr))   
    #Autovectores---
    x = list(range(n))
    y = list(range(0,2*n,2))
    vc = q[np.ix_(x,y)]
    return w,vr,sr,dr,vc,wr

#Leyenda:
    
    # w = Autovalores caso no amortiguado
    # vr = Autovector caso no amortiguado
    # sr = Autovalores caso amortiguado sin eliminar la parte imaginaria 
    # wr = Autovalores caso amortiguado
    # dr = factores de amortiguamiento crítico en caso amortiguado
    # vc = Autovector caso amortiguado


#Parte de calculo de las FRF´s:------
def simF(M,C,K,v_act,wt):
    # carga las matrices iniciales para hacer un sistema y mas adelante un espacio de 
    # estados parecido al que se hizo mas arriba pero en este caso solo va a querer una
    # salida pues para las FRF´s se actua en i (v_act) y mira en j, por la que se 
    # veran modificadas estas matrices para adaptarlas a esta parte.
      #GDL y matriz modelo vacia:
    n = M.shape[0]
    Z = np.zeros((n,n))
    z = np.zeros((n,1))
    H = np.zeros((wt.shape[0],n),dtype=complex) #Matriz para cargar los valores de las diferentes H (Receptancias)
    #Vector que indica la posicion (i) sobre la que actuar:
    v = np.zeros((n,1))
    if v_act > 1:
        v[v_act-1][:] = 1
    if v_act == 1:
        v[v_act-1] = 1
    #Matriz identidad:
    Im = np.eye(n)
    #Matriz iversa de M:
    M_Inv = linalg.inv(M)
    #Completa las matrices A y B como se hizo antes:
    A1 = np.concatenate((Z, Im), axis=1)
    A2 = np.concatenate(((-M_Inv@K), (-M_Inv@C)), axis=1)
    A = np.concatenate((A1, A2), axis=0)
    B = np.concatenate((z, (M_Inv@v)), axis=0) 
         # A la matriz B la multiplica abajo por
         #  un vector que sera de ceros menos un uno en
         #  la posicion que estudiaremos  
    #La matriz D siempre cero:                                 
    D = 0                                  
         # Al igual que antes, es cero pues no interesan las 
         # aceleraciones ya que de la movilidad sacaremos acelerancias
         # e inertancias y solo de 1x1 pues se va a estudiar de uno en uno.
    #La matriz C la varia según las salidas que se quiera:
    for a in range(0,n):
        C = np.zeros((1,2*n))
        C[0][a] = 1 # La matriz C tiene 1 en la diagonal de la primera
                    # parte (posiciones) y cero en la segunda (velocidades)
                    # pues solo interesa los deplazamientos (movilidad)
        ss = signal.StateSpace( A , B , C , D )
        wt,H1 = signal.freqresp(ss,wt) #H sera la amplitud que da la FRF
        #Construir las matrices que contienen los output de las FRF´s:
        H[:,a] = H1[:]
    return H

#Matriz para pasar de Receptancias a Movilidad y a acelerancias:
def H2V_V2A(H, wt):
    n = H.shape[1]
    j = H.imag
    V = np.zeros((H.shape),dtype=complex)
    A = np.zeros((H.shape),dtype=complex)
    for a in range(0,n):
        V[:,a] = j[:,a]*wt*H[:,a]
        A[:,a] = j[:,a]*wt*V[:,a]
    return V, A

